package geo;

import java.awt.Graphics;

/**
 * Courbe de (f(t), g(t))
 */
public class CourbeFonction2 extends ObjetGeo {
	Fonction2 f;
/** valeur minimale du paramètre t. */
	double tmin;
/** valeur maximale du paramètre t. */
	double tmax;
/** incrément. */
	double tincrement;

/**
 * Définit la courbe à partir d'une fonction
 *
 * @param f fonction (f((), g(t)) à définir
 * @param tmin valeur minimale du paramètre
 * @param tmax valeur maximale du paramètre
 * @param tincrement valeur à ajouter au paramètre à chaque étape du tracé
 */
	public CourbeFonction2 (Fonction2 f, double tmin, double tmax, double tincrement) {
		super();
		this.f = f;
		this.tmin = tmin;
		this.tmax = tmax;
		this.tincrement = tincrement;
	}

/**
 * Met à jour la courbe
 * @param f fonction (f((), g(t)) à définir
 * @param tmin valeur minimale du paramètre
 * @param tmax valeur maximale du paramètre
 * @param tincrement valeur à ajouter au paramètre à chaque étape du tracé
 */
	public void MAJ (Fonction2 f, double tmin, double tmax, double tincrement) {
		this.f = f;
		this.tmin = tmin;
		this.tmax = tmax;
		this.tincrement = tincrement;
	}

/**
 * Trace la courbe
 *
 * @param nom nom de la courbe
 * @param R repère associé au graphique
 * @param g support du tracé
 */
	public void trace (String nom, Repere R, Graphics g) {
		int Xp = Integer.MIN_VALUE;
		int Yp = Integer.MIN_VALUE;
		double ts2 = 0.5 * (tmax - tmin);
		for (double t = tmin; t < tmax; t += tincrement) {
			int X = Integer.MIN_VALUE;
			int Y = Integer.MIN_VALUE;
			double[] d = new double [2];
			if (f.defini(t)) {
				d = f.Image (t);
				X = R.X (d[0]);
				Y = R.Y (d[1]);
			} else {
				X = Integer.MIN_VALUE;
				Y = Integer.MIN_VALUE;
			}
			if ((Xp >= 0) && (Xp < R.XMAX) && (Yp >= 0) && (Yp < R.YMAX)
					&& (X >= 0) && (X < R.XMAX) && (Y >= 0) && (Y < R.YMAX)) {
				g.drawLine (Xp, Yp, X, Y);
				if ((t > ts2) && (nom != null) && (nom.length() != 0)) {
					g.drawString (nom, X + 3, Y - 3);
					ts2 = Double.MAX_VALUE;
				}
			}
			Xp = X;
			Yp = Y;
		}
	}

}
