package geo;

import java.awt.Graphics;

/**
 * Droite d'équation a x + b y + c = 0.<br />
 * a, b, c ne sont pas uniques mais la droite est définie dès que a et b ne sont pas tous nuls.
 */
public class Droite extends ObjetGeo {
/** Coefficient a dans l'équation de la droite ax + by + c = 0. */
	public double a;
/** Coefficient b dans l'équation de la droite ax + by + c = 0. */
	public double b;
/** Coefficient c dans l'équation de la droite ax + by + c = 0. */
	public double c;

/** Construit une droite nomnée non définie. */
	public Droite() {
		super();
	}

/** Construit la copie d'une droite */
	public Droite (Droite d) {
		a = d.a;
		b = d.b;
		c = d.c;
		defini = (a != 0.0) || (b != 0.0);
	}

/** Construit la droite passant par les points A et B */
	public Droite (Pt A, Pt B) {
		a = B.y - A.y;
		b = A.x - B.x;
		c = A.y * B.x - A.x * B.y;
		defini = ((a != 0.0) || (b != 0.0)) && A.defini && B.defini;
	}

/** Construit la droite passant par le point A et de vecteur directeur u */
	public Droite (Pt A, Vecteur u) {
		a = u.y;
		b = - u.x;
		c = A.y * u.x - A.x * u.y;
		defini = A.defini && u.defini && (u.x != 0.0 || u.y != 0.0);
	}

/** Construit la droite à partir des coordonnées de deux points */
	public Droite (double xA, double yA, double xB, double yB) {
		a = yB - yA;
		b = xA - xB;
		c = yA * xB - xA * yB;
		defini = ((a != 0.0) || (b != 0.0));
	}

/** Construit la droite à partir des coefficients a, b, c */
	public Droite (double a, double b, double c) {
		this.a = a;
		this.b = b;
		this.c = c;
		defini = ((a != 0.0) || (b != 0.0));
	}

/** Construit la droite à partir du segment s */
	public Droite (Segment s) {
		a = s.By - s.Ay;
		b = s.Ax - s.Bx;
		c = s.Ay * s.Bx - s.Ax * s.By;
		defini = ((a != 0.0) || (b != 0.0)) && s.defini;
	}

/** Met à jour la droite passant par le point A et de vecteur directeur u */
	public void MAJ (Pt A, Vecteur u) {
		a = u.y;
		b = - u.x;
		c = A.y * u.x - A.x * u.y;
		defini = A.defini && u.defini && (u.x != 0.0 || u.y != 0.0);
	}

/** Met à jour la droite à partir des points A et B */
	public void MAJ (Pt A, Pt B) {
		a = B.y - A.y;
		b = A.x - B.x;
		c = A.y * B.x - A.x * B.y;
		defini = ((a != 0.0) || (b != 0.0)) && A.defini && B.defini;
	}

/** Met à jour la copie de la droite d */
	public void MAJ (Droite d) {
		a = d.a;
		b = d.b;
		c = d.c;
		defini = d.defini;
	}

/** Met à jour la droite à partir des coordonnées de deux points de cette droite */
	public void MAJ (double xA, double yA, double xB, double yB) {
		a = yB - yA;
		b = xA - xB;
		c = yA * xB - xA * yB;
		defini = ((a != 0.0) || (b != 0.0));
	}

/** Met à jour la droite à partir des trois coefficents a, b, c */
	public void MAJ (double a, double b, double c) {
		this.a = a;
		this.b = b;
		this.c = c;
		defini = ((a != 0.0) || (b != 0.0));
	}

/** Retourne la droite parallèle à la droite courante et passant par le point A */
	public Droite parallele (Pt A) {
		Droite d = new Droite (a, b, - (a * A.x + b * A.y));
		d.defini = d.defini && A.defini;
		return d;
	}

/** Retourne la droite parallèle au segment s et passant par le point A */
	public Droite parallele (Segment s, Pt A) {
		double a = s.By - s.Ay;
		double b = s.Ax - s.Bx;
		Droite d = new Droite (a, b, - (a * A.x + b * A.y));
		d.defini = ((a != 0.0) || (b != 0.0)) && A.defini && s.defini;
		return d;
	}

/** Retourne la droite perpendiculaire à la droite courante et passant par le point A */
	public Droite perpendiculaire (Pt A) {
		Droite d = new Droite (- b, a, b * A.x - a * A.y);
		d.defini = d.defini && A.defini;
		return d;
	}

/** Retourne la droite perpendiculaire au segment s passant par le point A */
	static public Droite perpendiculaire (Segment s, Pt A) {
		double a = s.Bx - s.Ax;
		double b = s.By - s.Ay;
		Droite d = new Droite (a, b, - (a * A.x + b * A.y));
		d.defini = ((a != 0.0) || (b != 0.0)) && s.defini && A.defini;
		return d;
	}

/** Retourne la droite médiatrice de [AB] */
	static public Droite mediatrice (Pt A, Pt B) {
		Droite d = new Droite (B.x - A.x, B.y - A.y, 0.5 * (A.x * A.x - B.x * B.x + A.y * A.y - B.y * B.y));
		d.defini = d.defini && A.defini && B.defini;
		return d;
	}

/** Retourne la droite bissectrice de l'angle ABC */
	static public Droite bissectrice (Pt A, Pt B, Pt C) {
		boolean defini = A.defini && B.defini && C.defini;
		if (defini) {
			double bcx = C.x - B.x;
			double bcy = C.y - B.y;
			double bc = Math.sqrt (bcx * bcx + bcy * bcy);
			double bax = A.x - B.x;
			double bay = A.y - B.y;
			double ba = Math.sqrt (bax * bax + bay * bay);
			bax = bc * bax;
			bay = bc * bay;
			bcx = ba * bcx;
			bcy = ba * bcy;
			Droite d = new Droite (B.x, B.y, B.x + bax + bcx, B.y + bay + bcy);
			d.defini = defini;
			return d;
		}
		return new Droite (0.0, 0.0, 0.0, 0.0);
	}

/** Retourne, s'il existe, le point intersection de la droite courante et de la droites d */
	public Pt intersection (Droite d) {
		double det = a * d.b - d.a * b;
		boolean defini = this.defini && d.defini && (det != 0.0);
		double x = 0.0;
		double y = 0.0;
		if (defini) {
			x = (d.c * b - c * d.b) / det;
			y = (d.a * c - a * d.c) / det;
		}
		return new Pt (x, y, defini);
	}

/** Normalise l'équation de la droite&nbsp;: c'est à dire remplace a, b, c par des coefficients proportionnels tels que a^2 + b^2 = 1 */
	public void normalise() {
		double m = Math.sqrt (a * a + b * b);
		if (m != 0.0) {
			a = a / m;
			b = b / m;
			c = c / m;
		}
	}

/**
 * Trace la droite courante
 *
 * @param nom nom de la droite
 * @param R repère associé
 * @param g support graphique
 */
	public void trace (String nom, Repere R, Graphics g) {
		int N, X1, Y1, X2, Y2;
		X1 = Y1 = X2 = Y2 = 0;
		if (defini) {
			if (Math.abs (b) >= Math.abs (a)) {
				X1 = 0;
				Y1 = R.Y (- (a * R.x (0) + c) / b);
				X2 = R.XMAX; 
				Y2 = R.Y (- (a * R.x (X2) + c) / b);
			} else {
				Y1 = 0;
				X1 = R.X (- (b * R.y (0) + c) / a);
				Y2 = R.YMAX;
				X2 = R.X (- (b * R.y (Y2) + c) / a);
			}
			g.drawLine (X1, Y1, X2, Y2);
			if (nom != null && nom.length() != 0) {
				int i;
				if (Math.abs (b) >= Math.abs (a)) {
					i = R.XMAX / 2;
					g.drawString (nom, i + 3, R.Y (- (a * R.x (i) + c) / b) - 3);
				} else {
					i = R.YMAX / 2;
					g.drawString (nom, R.X (- (b * R.y (i) + c) / a) + 3, i - 3);
				}
			}
		}
	}

/** toString */
	public String toString () {
		return "Droite : a = " + a + " b = " + b + " c = " + c + " defini =" + defini;
	}

}
