package geo;

import java.awt.Graphics;

/**
 * Segment [AB]
 */
public class Segment extends ObjetGeo {
/** Abscisse de l'extrémité A du segment */
	public double Ax;
/** Ordonnée de l'extrémité A du segment */
	public double Ay;
/** Abscisse de l'extrémité B du segment */
	public double Bx;
/** Ordonnée de l'extrémité B du segment */
	public double By;

/** Construit un segment non défini */
	public Segment() {
		super();
	}

/** Construit le segment [AB] */
	public Segment (Pt A, Pt B) {
		MAJ (A, B);
	}

/** Construit le segment à partir des coordonnées des extrémités */
	public Segment (double Ax, double Ay, double Bx, double By) {
		this.Ax = Ax;
		this.Ay = Ay;
		this.Bx = Bx;
		this.By = By;
		defini = true;
	}

/** Met à jour le segment à partir des extrémités */
	public void MAJ (Pt A, Pt B) {
		Ax = A.x;
		Ay = A.y;
		Bx = B.x;
		By = B.y;
		defini = A.defini && B.defini;
	}

/** Met à jour le segment à partir des coordonnées des extrémités */
	public void MAJ (double Ax, double Ay, double Bx, double By) {
		this.Ax = Ax;
		this.Ay = Ay;
		this.Bx = Bx;
		this.By = By;
		defini = true;
	}

/** Retourne la droite nommée passant par A et parallèle au segment courant */
	public Droite parallele (Pt A) {
		double a = By - Ay;
		double b = Ax - Bx;
		Droite d = new Droite (a, b, - (a * A.x + b * A.y));
		d.defini = ((a != 0.0) || (b != 0.0)) && A.defini && defini;
		return d;
	}

/** Retourne la droite passant par A et perpendiculaire au segment courant */
	public Droite perpendiculaire (Pt A) {
		double a = Bx - Ax;
		double b = By - Ay;
		Droite d = new Droite (a, b, - (a * A.x + b * A.y));
		d.defini = ((a != 0.0) || (b != 0.0)) && A.defini && defini;
		return d;
	}

/** Retourne la droite prolongeant le segment courant */
	public Droite droite() {
		double a = By - Ay;
		double b = Ax - Bx;
		Droite d = new Droite (a, b, Ay * Bx - Ax * By);
		d.defini = ((a != 0.0) || (b != 0.0)) && defini;
		return d;
	}


/** Retourne la longueur du segment courant */
	public double longueur() {
		if (defini) {
			double u = Bx - Ax;
			double v = By - Ay;
			return Math.sqrt (u * u + v * v);
		} else
			return Double.NaN;
	}

/**
 * Trace le segment courant
 *
 * @param nom nom du segment
 * @param R repère associé
 * @param g support graphique
 */
	public void trace (String nom, Repere R, Graphics g) {
		if (defini) {
			int AX = R.X (Ax);
			int AY = R.Y (Ay);
			int BX = R.X (Bx);
			int BY = R.Y (By);
			g.drawLine (AX, AY, BX, BY);
			if (nom != null && nom.length() != 0)
				g.drawString (nom, (AX + BX) / 2 + 3, (AY + BY) / 2 - 3);
		}
	}

/** toString */
	public String toString() {
		return "Segment : [AB] A(" + Ax + ", " + Ay + ") B(" + Bx + ", " + By + ") defini = " +defini;
	}

}
